@@ -10,15 +10,24 @@ What
 4) Also function evaluation from other config values (not only static value stored).
 5) Options hierarchy (nested options).
 
+
 Examples:
 =========
 
+    By default, config are type validated, so if you configure bad type, error will be raised. You can define
+    variables as usual, or can use property for using setters for dynamic behavior. It's recommended to use
+    'MyProperty' as it's type validated and there is no need for defining getter.
+    
     >>> from __future__ import annotations
     >>> from typing_extensions import Literal
     ...
     >>> class SimpleConfig(Config):
+    ...
+    ...     simple_var: Literal[1, 2, 3] = 1  # Variables are validated including Unions and Literal
+    ...     '''You can document it like this'''
+    ...
     ...     @MyProperty
-    ...     def var(self) -> int:  # Type hints are validated.
+    ...     def dynamic_var(self) -> int:  # Type hints are validated.
     ...         '''
     ...         Type:
     ...             int
@@ -27,41 +36,39 @@ Examples:
     ...             123
     ...
     ...         This is docstrings (also visible in IDE, because not defined dynamically).
-    ...         Also visible in Sphinx documentation.'''
+    ...         Also visible in Sphinx documentation. It's recommended to document a type for sphinx docs.'''
     ...
-    ...         return 123  # This is initial value that can be edited.
-    ...
-    ...     @MyProperty
-    ...     def var_literal(self) -> Literal[1, 2, 3]:  # Literal options are also validated
-    ...         return 2
-    ...
-    ...     @MyProperty   # If other defined value is changed, computed property is also updated
-    ...     def evaluated(self) -> int | float:
-    ...         return self.var + 1
+    ...         return self.simple_var + 10  # This is default value that can be edited.
     ...
     >>> config = SimpleConfig()
-    >>> config.var
-    123
-    >>> config.var = 665
-    >>> config.var
-    665
-    >>> config['var']  # You can also access params as in a dictionary
-    665
-    >>> config.var = "String is problem"
+    >>> config.simple_var
+    1
+    >>> config.simple_var = 2
+    >>> config.simple_var
+    2
+    >>> config['simple_var']  # You can also access params as in a dictionary
+    2
+    >>> config.simple_var = "String is problem"  # doctest: +3.8
     Traceback (most recent call last):
     TypeError: ...
     ...
-    >>> config.var_literal = 4
+    >>> config.simple_var = 4  # Literal is also validated  # doctest: +3.8
     Traceback (most recent call last):
     TypeError: ...
     ...
-    >>> config.evaluated
-    666
+    >>> config.simple_var = 2  # Restoring value as on 3.7 it's no validated, so test would fails
+    >>> config.dynamic_var
+    12
 
     You can still setup a function (or lambda expression) as a new value
     and returned value still will be validated
 
-    >>> config.var = lambda self: self.var_literal + 1
+    >>> config.dynamic_var = lambda self: self.simple_var + 100
+    >>> config.dynamic_var
+    102 
+    >>> config.dynamic_var = lambda self: "String is problem"  # doctest: +3.8
+    Traceback (most recent call last):
+    TypeError: ...
 
 This is how help looks like in VS Code
 
@@ -75,51 +82,45 @@ Hierarchical config
 -------------------
 
 It is possible to use another config object as a value in config and thus hierarchical configs can be created.
-Note:
-    Use unique values for all config variables even if they are in various subconfig.
 
->>> from mypythontools.config import Config as mypythontools_config
+Note:
+    Use unique values for all config variables even if they are in various subconfig!
+
+>>> from mypythontools.config import MyProperty, Config as ConfigBase
 ...
->>> class Config(mypythontools_config):
+>>> class Config(ConfigBase):
 ...     def __init__(self) -> None:
 ...         self.subconfig1 = self.SubConfiguration1()
 ...         self.subconfig2 = self.SubConfiguration2()
 ...
-...     class SubConfiguration1(Config):
+...     class SubConfiguration1(ConfigBase):
 ...         def __init__(self) -> None:
 ...             self.subsubconfig = self.SubSubConfiguration()
 ...
-...         class SubSubConfiguration(Config):
-...             @MyProperty
-...             def value1(self) -> Literal[0, 1, 2, 3]:
-...                 '''Documentation here
+...         class SubSubConfiguration(ConfigBase):
 ...
-...                 Options: [0, 1, 2, 3]
-...                 '''
-...                 return 3
+...             value1: Literal[0, 1, 2, 3] = 3
 ...
 ...             @MyProperty
-...             def value2(self):
+...             def subconfig_value(self):
 ...                 return self.value1 + 1
 ...
-...     class SubConfiguration2(Config):
+...     class SubConfiguration2(ConfigBase):
 ...         @MyProperty
 ...         def other_val(self):
-...             return self.value2 + 1
+...             return self.subconfig_value + 1
 ...
-...     # Also subconfig category can contain values itself
-...     @MyProperty
-...     def value3(self) -> int:
-...         return 3
+...     # Also main category can contain values itself
+...     value3: int = 3
 ...
 >>> config = Config()
 ...
->>> config.subconfig1.subsubconfig.value2
+>>> config.subconfig1.subsubconfig.subconfig_value
 4
 
 You can access value from config as well as from subcategory
 
->>> config.value2
+>>> config.subconfig_value
 4
 
 Copy
@@ -128,7 +129,7 @@ Copy
 Sometimes you need more instances of settings and you need copy of existing configuration.
 Copy is deepcopy by default.
 
->>> config2 = config.copy()
+>>> config2 = config.do.copy()
 >>> config2.value3 = 0
 >>> config2.value3
 0
@@ -140,10 +141,10 @@ Bulk update
 
 Sometimes you want to update many values with flat dictionary.
 
->>> config.update({'value3': 2, 'value1': 0})
+>>> config.do.update({'value3': 2, 'value1': 0})
 >>> config.value3
 2
->>> config.update({"not_existing": "Should fail"})
+>>> config.do.update({"not_existing": "Should fail"})
 Traceback (most recent call last):
 AttributeError: ...
 
@@ -152,15 +153,22 @@ Get flat dictionary
 
 There is a function that will export all the values to the flat dictionary (no dynamic anymore, just values).
 
->>> config.get_dict()
-{'value3': 2, 'value1': 0, 'value2': 1, 'other_val': 2}
+>>> flattened = config.do.get_dict()
+>>> flattened
+{'value3': 2, 'value1': 0, 'subconfig_value': 1, 'other_val': 2}
+
+>>> another_config = Config()
+>>> another_config.do.update(flattened)
+>>> another_config.subconfig1.subsubconfig.value1
+0
+
+Convert dict (or JSON) back to config
 
 Reset
 -----
-You can reset to default values
+You can reset to default values like this
 
->>> config.value1 = 1
->>> config.reset()
+>>> config = Config()
 >>> config.value1
 3
 
@@ -173,7 +181,7 @@ CLI is provided by argparse. When using `with_argparse` method, it will
 
 ::
 
-    config.with_argparse()
+    config.do.with_argparse()
 
 Now you can use in terminal like.
 
