@@ -61,6 +61,6 @@ Examples:
     >>> config.var = lambda self: self.var_literal + 1
 """
 
-from mypythontools.property.property_internal import MyProperty, init_my_properties
+from mypythontools.property.property_internal import MyProperty, MyPropertyClass, init_my_properties
 
-__all__ = ["MyProperty", "init_my_properties"]
+__all__ = ["MyProperty", "MyPropertyClass", "init_my_properties"]
